﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using System.Collections.Generic;
using System.Data;
using System.Linq;

using Npgsql;

using Dapper;

using TestProject.Models;

namespace TestProject.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class CodeController : ControllerBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region INQUIRY_LIST_SQL

        /// <summary>
        /// INQUIRY_LIST_SQL
        /// </summary>
        private static string INQUIRY_LIST_SQL = @"
SELECT
    A.ID            AS ID
   ,A.GROUP_ID      AS GROUP_ID
   ,A.NAME          AS NAME
   ,A.VALUE         AS VALUE
   ,A.DISPLAY_ORDER AS DISPLAY_ORDER
   ,A.STATUS_CODE   AS STATUS_CODE
FROM     COM_CODE A
WHERE    A.GROUP_ID = @GROUP_ID
ORDER BY A.DISPLAY_ORDER ASC
";

        #endregion
        #region INQUIRY_ITEM

        /// <summary>
        /// INQUIRY_ITEM_SQL
        /// </summary>
        private static string INQUIRY_ITEM_SQL = @"
SELECT
    A.ID            AS ID
   ,A.GROUP_ID      AS GROUP_ID
   ,A.NAME          AS NAME
   ,A.VALUE         AS VALUE
   ,A.DISPLAY_ORDER AS DISPLAY_ORDER
   ,A.STATUS_CODE   AS STATUS_CODE
FROM  COM_CODE A
WHERE A.ID = @ID
";

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 구성
        /// </summary>
        private readonly IConfiguration configuration;

        /// <summary>
        /// 로거
        /// </summary>
        private readonly ILogger<CodeController> logger;

        /// <summary>
        /// 연결 문자열
        /// </summary>
        private readonly string connectionString;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CodeController(configuration, logger)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="configuration">구성</param>
        /// <param name="logger">로거</param>
        public CodeController(IConfiguration configuration, ILogger<CodeController> logger)
        {
            this.configuration    = configuration;
            this.logger           = logger;
            this.connectionString = configuration.GetConnectionString("DefaultConnection");
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 리스트 조회하기 - InquiryList(groupID)

        /// <summary>
        /// 리스트 조회하기
        /// </summary>
        /// <param name="groupID">그룹 ID</param>
        /// <returns>리스트</returns>
        [HttpGet]
        [Route("InquiryList")]
        public IEnumerable<CodeModel> InquiryList(string groupID)
        {
            using(IDbConnection connection = CreateConnection())
            {
                var result = connection.Query<CodeModel>(INQUIRY_LIST_SQL, new CodeKeyModel { GROUP_ID = groupID });

                return result;
            }
        }

        #endregion
        #region 항목 조회하기 - InquiryItem(id)

        /// <summary>
        /// 항목 조회하기
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns></returns>
        [HttpGet]
        [Route("InquiryItem")]
        public CodeModel InquiryItem(string id)
        {
            using(IDbConnection connection = CreateConnection())
            {
                var result = connection.Query<CodeModel>(INQUIRY_ITEM_SQL, new CodeKeyModel { ID = id });

                return result.FirstOrDefault();
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 연결 생성하기 - CreateConnection()

        /// <summary>
        /// 연결 생성하기
        /// </summary>
        /// <returns>연결</returns>
        private IDbConnection CreateConnection()
        {
            return new NpgsqlConnection(this.connectionString);
        }

        #endregion
    }
}